#-----------------------------------------------------------------------------
# Project    : Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File       : sim.tcl
# Authors    : Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
# Lab        : GRM - Polytechnique Montréal
# Date       : <2019-07-24 Wed>
#-----------------------------------------------------------------------------
# Description: Script pour les simulations du compteur BCD avec Modelsim
# HOW-TO     : source setup.csh && cd simulation/
#            : (CLI) > vsim -c -do ${SCRIPTS_DIR}/sim.tcl
#            : (GUI) > vsim -do ${SCRIPTS_DIR}/sim.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_BEH  0;  # Modèle comportemental
set DO_SYN  0;  # Netlist post-synthèse
set DO_PNR  1;  # Netlist post-placement-routage
set DO_SIM  1;  # 0=Compilation; 1=Compilation + Simulation

set top   compteur
set tb    compteur_tb
set dut   dut
set work  work
set lib   gpdk45

if { $DO_BEH } { set workD $::env(SIM_DIR)/beh }
if { $DO_SYN } { set workD $::env(SIM_DIR)/syn }
if { $DO_PNR } { set workD $::env(SIM_DIR)/pnr }

set log  ${workD}/${top}.log
set wlf  ${workD}/${top}.wlf
set vcd  ${workD}/${top}.vcd
set wave ${workD}/${top}.wave.do

if { $DO_SYN } { set net $::env(SYN_NET_DIR)/${top}.syn   }
if { $DO_PNR } { set net $::env(PNR_NET_DIR)/${top}.route }

#-----------------------------------------------------------------------------
# COMPILATION
#-----------------------------------------------------------------------------

# Création du répertoire de travail
if { ![file exist $workD] } {
    file mkdir $workD
}

# Mise à jour du modelsim.ini avec les librairies GPDK45
if { [file exist "./modelsim.ini"] } {
    file delete "./modelsim.ini"
}
vmap -c
vmap $lib $::env(KIT_SIMLIB)/gsclib045_slow

# Work
if { [file exist ${workD}/${work}] } {
    vdel -all -lib ${workD}/${work}
}
vlib ${workD}/${work}
vmap ${work} ${workD}/${work}

# Model
if { $DO_BEH } {
    vcom -2008 -work $work $::env(SRC_DIR)/${top}.vhd
    vcom -2008 -work $work $::env(SRC_DIR)/${tb}.vhd

# Netlist
} elseif { $DO_SYN || $DO_PNR } {
    vlog -work $work ${net}.v
    vcom -2008 -work $work $::env(SRC_DIR)/${tb}.vhd
}

#-----------------------------------------------------------------------------
# SIMULATION
#-----------------------------------------------------------------------------

if { $DO_SIM && $DO_BEH } {
    vsim -c -t ps -voptargs=+acc -do run.do -logfile $log -wlf $wlf ${work}.${tb}
}

if { $DO_SIM && ($DO_SYN || $DO_PNR) } {
    vsim -c -t ps -voptargs=+acc -do run.do -logfile $log -wlf $wlf -L $lib -sdfmax ${dut}=${net}.sdf ${work}.${tb}
}
